/* 
14/03/2023
Projet ProTis : Asservissement d'une lampe frontale 
Fanny Guiraud, Lucas Prevotel, (Bastien La Torre), Ines Vergara, Tony Stark, Harry potter, Booba, Mère théresa, Bob marley et 30 millions d'amis.

Matériel nécessaire :
- Nucleo L476RG
- Bandeau de 6 LEDs
- LED de puissance luxeon k2
- 3 Photodiodes
- matériel électronique : résistances et transistor

Avec les bons câblages ce code permet de éclairer ou non suivant différents modes (éteint, diffus et étroit).   

*/

#include "mbed.h" 
#include "PixelArray.h"
#include "WS2812.h"

#define NB_LED 6     // nombre de LED sur le bandeau, ici 2*3 (2 premières zone ouest, 2 suivantes zone nord, 2 dernières zone est)
#define min  2       // seuil minimal de luminosité accepté
#define max  2.8     // seuil maximal de luminosité accepté (au delà plus besoin d'éclairer, la luminosité ambiante est suffisante)
#define palier 10    // paliers de luminosité (on incrémente l'éclairage des LED de 10 en 10)

PixelArray px(10);   // tableau de 10 pixels de 3 octets RGB modifié par l'asservissement
WS2812 bandeau(D9, NB_LED, 3, 12, 9, 12); // pilote le bandeau de LED (bLED) branché en D9, valeurs suivantes données par constructeurs donc blc
InterruptIn bouton(D4); 
PwmOut led(D11);       // sortie vers la led de puissance                          
AnalogIn pdouest(A2);  // info venant de la photodiode ouest
AnalogIn pdnord(A1);   // info venant de la photodiode nord
AnalogIn pdest(A0);    // info venant de la photodiode est

int mode=0;        // prendra les valeurs 0,1 ou 2 correspondant au mode à actionner 
int numero;        // numero permet de parcourir tout le tableau px pour modifier toutes les LEDS
double pdO;        // luminosité mesurée par la photodiode ouest   
double pdN;        // luminosité mesurée par la photodiode nord   
double pdE;        // luminosité mesurée par la photodiode est   
int o = 100;       // luminosité d'une des leds ouest, cf suite
int n = 100;       // luminosité d'une des leds nord, cf suite
int e = 100;       // luminosité d'une des leds est, cf suite
void bouton_changemode(void); // fonction d'interruption pour changer de mode quand on appuie sur le bouton
void bandeau_fonction(void);  // fonction commandant l'asservissement du bandeau 
void bandeau_eteint(void);    // fonction permettant d'éteindre le bandeau

double vo[10];  // Declare le tableau de 10 valeurs sur lesquelles on va faire la moyenne 
double vn[10];
double ve[10];
int i=0;        // compteur moyenne glissante

void bouton_changemode(){
    mode++;          // on veut changer de mode 
    if (mode==3){ 
        mode=0; 
    } 
    if (mode==0){         // lampe est en mode étroit et doit s'éteindre 
        bandeau_eteint(); // on éteint le bandeau
        led.write(0);     // consigne pour éteindre la lampe 
    } 
    else if (mode==1){    // lampe est éteinte et doit se mettre en diffus 
        bandeau_fonction();
        led.write(0);
    } 
    else if (mode==2){ // lampe est en mode diffus et doit se mettre en étroit 
        bandeau_eteint();
        led.write(1);
    }
    wait_us(500000);   // optimal pour éviter de sauter des modes en restant appuyé un chouïa trop longtemps
}

void bandeau_eteint(){ 
    for (numero = 0; numero < NB_LED; numero++) {   // on éteint toutes les LEDS du bandeau
        px.SetR(numero, 0);
        px.SetG(numero, 0);
        px.SetB(numero, 0);               // la led numero "numero" s'éteint
    }  
    bandeau.write(px.getBuf());  // Mise à jour de la matrice
}

void bandeau_fonction(){
    if (mode==1){
        if (pdO<min){            // besoin d'asservir led ouest si pdO pas dans l'intervalle
            if (o < 250) {       // si pas déjà au max...       
                o = o+palier;    // luminosité de la led numero "numero"            
                for(numero=0; numero<2 ;numero++){     // section ouest : leds 1 2 et 3
                    px.SetR(numero, o);                // ...on l'augmente d'un palier 
                    px.SetG(numero, o);
                    px.SetB(numero, o);
                }
            }
        }
        if (pdO>max){
            if (o != 0) {          // si pas déjà à 0...
                o = o-palier;      // luminosité de la led numero "numero"           
                for(numero=0; numero<2 ;numero++){    // section ouest : leds 1 2 et 3
                    px.SetR(numero, o);               // ...on le diminue d'un palier 
                    px.SetG(numero, o);
                    px.SetB(numero, o);
                }
            }
        }

        if (pdN<min){            // besoin d'asservir led nord si pdN pas dans l'intervalle
            if (n < 250) {       // si pas déjà au max...
                n = n+palier;    // luminosité de la led numero "numero"
                for(numero=2; numero<4 ;numero++){     // section ouest : leds 4 5 et 6
                    px.SetR(numero, n);                // ...on l'augmente d'un niveau 
                    px.SetG(numero, n);
                    px.SetB(numero, n);
                }
            }        
        }
        if (pdN>max){   
            if (n != 0) {            // si pas déjà à 0...    
                n = n-palier;        // luminosité de la led numero "numero"
                for(numero=2; numero<4 ;numero++){      // section ouest : leds 4 5 et 6
                    px.SetR(numero, n);                 // ...on la diminue d'un niveau 
                    px.SetG(numero, n);
                    px.SetB(numero, n);
                }
            } 
        }

        if (pdE<min){             // besoin d'asservir led est si pdE pas dans l'intervalle
            if (e < 250) {        // si pas déjà au max...
                e = e+palier;     // luminosité de la led numero "numero"
                for(numero=4; numero<6 ;numero++){      // section ouest : leds 1 2 et 3
                    px.SetR(numero, e);                 // ...on l'augmente d'un niveau 
                    px.SetG(numero, e);
                    px.SetB(numero, e);
                }
            } 
        }
        if (pdE>max){   
            if (e != 0) {            // si pas déjà à 0...
                e = e-palier;        // luminosité de la led numero "numero"
                for(numero=4; numero<6 ;numero++){      // section ouest : leds 1 2 et 3
                    px.SetR(numero, e);                 // ...on le diminue d'un niveau 
                    px.SetG(numero, e);
                    px.SetB(numero, e);
                }
            }
        }
    }
    bandeau.write(px.getBuf());  // Mise à jour de la matrice
    wait_us(10000);
    }

int main(){ 
    bandeau.useII(WS2812::GLOBAL);  // Initialisation de la liaison avec les LEDs
    bandeau.setII(255);             // Luminosité maximale à 255/255 
    // Réinitialisation écran
    for (numero = 0; numero < NB_LED; numero++) {   //on éteint toutes les LEDS du bandeau
        px.SetR(numero, 0);
        px.SetG(numero, 0);
        px.SetB(numero, 0);                // la LED numero "numero" 'éteint
    }  

    bouton.rise(&bouton_changemode);
    led.period_ms(1);  //pcq la led de puissance clignotait

    bandeau.write(px.getBuf());       // Mise à jour de la matrice : getBuf récupère toutes les valeurs de px

    
    while (1) { 
    vo[i]=pdouest.read()*3.3;   // on initialise la liste avec les 10 premieres valeurs 
    vn[i]=pdnord.read()*3.3;    // on initialise la liste avec les 10 premieres valeurs 
    ve[i]=pdest.read()*3.3;     // on initialise la liste avec les 10 premieres valeurs 
    i=i+1; 
    pdO=(vo[0]+vo[1]+vo[2]+vo[3]+vo[4]+vo[5]+vo[6]+vo[7]+vo[8]+vo[9])/10;    // lorsque la 11 arrive on fait la moyenne des 10 premieres 
    pdN=(vn[0]+vn[1]+vn[2]+vn[3]+vn[4]+vn[5]+vn[6]+vn[7]+vn[8]+vn[9])/10;
    pdE=(ve[0]+ve[1]+ve[2]+ve[3]+ve[4]+ve[5]+ve[6]+ve[7]+ve[8]+ve[9])/10;
    if(i==11) { 
        i=0;         // on remet notre compteur à 0
    }

        if (mode==1){
            bandeau_fonction();
        }
        else {
            bandeau_eteint();
        }
    }
} 