/* 
14/03/2023
Projet ProTis : Asservissement d'une lampe frontale 
Fanny Guiraud, Lucas Prevotel, (Bastien La Torre), Ines Vergara, Tony Stark, Harry potter, Booba, Mère théresa, Bob marley et 30 millions d'amis.

Matériel nécessaire :
- Nucleo L476RG
- Bandeau de 9 LEDs
- 3 photodiodes
- matériel électronique : résistances et transistor

Avec les bons câblages ce code permet d'asservir en fonction de la luminosité captée par les photodiodes, les 3 LEDs des sections correspondant à chaque photodiode

*/


#include "mbed.h" 
#include "PixelArray.h"
#include "WS2812.h"

#define NB_LED 9 // nombre de LED sur le bandeau, ici 3*3 (3 premières zone ouest, 3 suivantes zone nord, 3 dernières zone est)
#define min      // seuil minimal de luminosité accepté
#define max      // seuil maximal de luminosité accepté
#define palier   // paliers de luminosité

PixelArray px(10); // tableau de 10 pixels de 3 octets RGB modifié par l'asservissement
WS2812 bandeau(D9, NB_LED, 3, 12, 9, 12); // pilote le bandeau de LED (bLED) branché en D9, valeurs suivantes données par constructeurs donc blc
Analog In pdouest() // info venant de la photodiode ouest
Analog In pdnord()  // info venant de la photodiode nord
Analog In pdest()   // info venant de la photodiode est


int main(){ 
    int j = 0;      // i permet de parcourir tout le tableau pour modifier toutes les LEDS
    int k = 0;      // valeur de luminosité comprisek entre 0 et 255 (équivalant respectivement à 0 et 100% de luminosité)
    int pdO;        // luminosité mesurée par la photodiode ouest   
    int pdN;        // luminosité mesurée par la photodiode nord   
    int pdE;        // luminosité mesurée par la photodiode est   
    int o = 100;    // luminosité d'une des leds ouest, cf suite
    int n = 100;    // luminosité d'une des leds nord, cf suite
    int e = 100;    // luminosité d'une des leds est, cf suite

    bandeau.useII(WS2812::GLOBAL);  // Initialisation de la liaison avec les LEDs
    bandeau.setII(0);               // Luminosité maximale à 0/255
    
    // Réinitialisation écran
    for (j = 0; j < NB_LED; j++) {   //on éteint toutes les LEDS du bandeau
        px.Set(j, 0);                // la ième LED s'éteint
    }  
    
    bandeau.write(px.getBuf());      // Mise à jour de la matrice : getBuf récupère toutes les valeurs de px
    bandeau.setII(30);               // Luminosité maximale à 30/255

    while(1) {
        pdO = read.pdouest ;         // on prend les nouvelles mesures des photodiodes
        pdN = read.pdnord ;
        pdE = read.pdest ;
        j=0;

        if (pdO<min){    // besoin d'asservir led ouest si pdO pas dans l'intervalle
            for(j=0; j<3 ;j++){     // section ouest : leds 1 2 et 3
                o = px.Get(j)       // luminosité de la ième led
                if o != 255 {       // si pas déjà au max...
                    px.Set(j, o+palier);      // ...on l'augmente d'un palier 
                }
            }
        }
        if (pdO>max){
            for(j=0; j<3 ;j++){    // section ouest : leds 1 2 et 3
                o = px.Get(j)      // luminosité de la ième led
                if o != 0 {        // si pas déjà à 0...
                    px.Set(j, o-palier);      // ...on le diminue d'un palier 
                }
            }
        }

        if (pdN<min){   // besoin d'asservir led nord si pdN pas dans l'intervalle
            for(j=3; j<6 ;j++){     // section ouest : leds 4 5 et 6
                n = px.Get(j)       // luminosité de la ième led
                if n != 255 {       // si pas déjà au max...
                    px.Set(j, n+palier);      // ...on l'augmente d'un niveau 
                }
            }
        }
        if (pdN>max){   
            for(j=3; j<6 ;j++){      // section ouest : leds 4 5 et 6
                n = px.Get(j)        // luminosité de la ième led
                if n != 0 {        // si pas déjà au max...
                    px.Set(j, n-palier);      // ...on l'augmente d'un niveau 
                }
            }
        }

        if (pdE<min){   // besoin d'asservir led est si pdE  pas dans l'intervalle
            for(j=6; j<9 ;j++){      // section ouest : leds 1 2 et 3
                e = px.Get(j)        // luminosité de la ième led
                if e != 255 {        // si pas déjà au max...
                px.Set(j, e+palier);      // ...on l'augmente d'un niveau 
                }
            }
        }
        if (pdE>max){   
            for(j=6; j<9 ;j++){      // section ouest : leds 1 2 et 3
                e = px.Get(j)        // luminosité de la ième led
                if e != 0 {        // si pas déjà au max...
                px.Set(j, e-palier);      // ...on l'augmente d'un niveau 
                }
            }
        }

        bandeau.write(px.getBuf());  // Mise à jour de la matrice

    }
} 